import os 
import typer

from kamaqi.utils.files import add_template_file
from kamaqi.utils.files import read_project_file
from kamaqi.utils.files import add_kamaqi_file
from kamaqi.app.settings import get_kamaqi_template


app=typer.Typer(help="Create files for your apps")

@app.command(name="apps",
             help="Create files for multiple apps")
def upgrade_apps():

    project_data = read_project_file()
    project_path = project_data["project_path"]
    project_type = project_data["project_type"]
    apps_list=[]
    
    for app, data in project_data["apps"].items():
        if data["status"] == "added":
            apps_list.append(app)

    base_dir_files=""
    if project_type=="normal":
        base_dir_files=f"{project_path}"
    else:
        base_dir_files=f"{project_path}/src"

    for app_name in apps_list:
        os.mkdir(f"{base_dir_files}/{app_name}/")

        for template_name in ["router", "schemas","crud"]:
            template = get_kamaqi_template(template_name)
            template_text=template.render(app=app_name[:-1])
            add_template_file(f"{base_dir_files}/{app_name}/{template_name}.py",template_text)
            
    template = get_kamaqi_template("main")
    template_text=template.render(**project_data)
    add_template_file(f"{base_dir_files}/main.py",template_text)

    new_apps = [app[:-1] for app in apps_list]

    template = get_kamaqi_template("model")
  
    with open(f"{base_dir_files}/database/models.py",mode="r") as f:
        models_text = f.read()
        f.close()

    for app in new_apps:
        template_text=template.render(app=app)
        models_text+="\n\n"+ template_text
    
    add_template_file(f"{base_dir_files}/database/models.py",models_text)

    for app_name in apps_list:
        project_data["apps"][app_name]["status"]="upgraded"

    add_kamaqi_file(f"{str(project_path)}/kamaqi.json",project_data)
    
    



from typer import Typer
from rich import print
from kamaqi.utils.files import read_project_file

app=Typer(help="Show apps, endpoints and models")

@app.command(name="apps",
             help="Show apps and their metadata")
def list_apps():
    
    project_file = read_project_file()
    project_apps = project_file["apps"]
    print(project_apps)

@app.command(name="config",
            help="Show kamaqi.json")
def show_config():
    project_file=read_project_file()
    print(project_file)



from pathlib import Path
from jinja2 import Environment, FileSystemLoader
from kamaqi.utils.files import read_project_file,save_project_file

def add_project_path(path:Path):
    data=read_project_file()
    data["project_path"]=str(path)
    save_project_file(data)

def get_project_path():
    data=read_project_file()
    return data["project_path"]

def add_app_to_project_config(app_name:str):

    data=read_project_file()
    if data["apps"]:
        data["apps"].append(app_name)
    else:
        data["apps"]=[app_name]
    save_project_file(data)

def get_project_apps():
    data=read_project_file()
    return data["apps"]

def update_project_main_apps():

    environment = Environment(loader=FileSystemLoader("kamaqi/templates"))
    template = environment.get_template("main.txt")
    project_apps=get_project_apps()
    content = template.render(apps=project_apps)

    project_dir=get_project_path()

    with open(f"{project_dir}/main.py", mode="w", encoding="utf-8") as main_file:
        main_file.write(content)
        print(f"update main")
        main_file.close()
    






import json
from pathlib import Path

def read_project_file():
    with open("kamaqi.json",mode="r") as f:
        data=json.load(f)
        f.close()
    
    return data

def save_project_file(data:dict):

    with open("kamaqi.json", "w") as outfile:
        json.dump(data, outfile,indent=4)
        outfile.close()

def create_empty_file(path:Path):

    with open(path, "w") as outfile:
        outfile.close()

def read_template_file(path:Path)->str:
    
    with open(path, "r") as outfile:
        text = outfile.read()
        outfile.close()

    return text

def add_template_file(path:Path,data:str):

    with open(path, "w") as outfile:
        outfile.write(data)
        outfile.close()

def add_kamaqi_file(path:Path,data:dict):

    with open(path, "w") as outfile:
        json.dump(data, outfile)
        outfile.close()


import os 
import subprocess
from rich import print
from typer import Typer
from pathlib import Path
from kamaqi.app.settings import get_kamaqi_template
from kamaqi.utils.files import add_template_file
from kamaqi.utils.files import add_kamaqi_file
from kamaqi.app.settings import choose_project_type
from kamaqi.app.settings import choose_database_type

app = Typer(help="Init and configure your project")

@app.command(name="project",
            help="Init your project")
def set_project_path(project_name:str):

    project_path = Path(f"{os.getcwd()}/{project_name}")
    project_type=choose_project_type()
    database_type=choose_database_type(project_type)

    secret_key= subprocess.run(['openssl', 'rand','-hex','32'], stdout=subprocess.PIPE)
    secret_key=secret_key.stdout.decode('utf-8').split('\n')[0]

    project_data={
        "project_path": str(project_path),
        "project_name":project_name,
        "project_type":project_type,
        "database_type":database_type,
        "secret_key":secret_key,
        "apps":{project_name:{"status":"added"}}
        }
    
    base_dir_files=""

    os.mkdir(project_path)
    if project_type == 'normal': 
        os.mkdir(f"{project_path}/{project_name}/")
        os.mkdir(f"{project_path}/database/")
        base_dir_files=project_path
    else:
        os.mkdir(f"{project_path}/src/")
        os.mkdir(f"{project_path}/src/{project_name}/")
        os.mkdir(f"{project_path}/src/database/")
        base_dir_files=f"{project_path}/src"

    env_template=get_kamaqi_template("env")
    env_text= env_template.render(**project_data)
    env_path=f"{base_dir_files}/.env"
    add_template_file(env_path, env_text)

    project_templates=["auth","router", "settings", "schemas", "exceptions","database"]

    for template_name in project_templates:
        template=get_kamaqi_template(f"app_{template_name}")
        template_text=template.render(**project_data)

        if template_name=="database":
            add_template_file(f"{base_dir_files}/database/{template_name}.py",template_text)
        else:
            add_template_file(f"{base_dir_files}/{project_name}/{template_name}.py",template_text)
          

    template=get_kamaqi_template("models")
    template_text=template.render(**project_data)
    add_template_file(f"{base_dir_files}/database/models.py",template_text)

    template=get_kamaqi_template("requirements")
    template_text=template.render(**project_data)
    template_text=template_text.replace("\n\n","\n")
    template_text=template_text.replace("\n\n","\n")
    add_template_file(f"{project_path}/requirements.txt",template_text)

    
    template=get_kamaqi_template("main")
    template_text=template.render(**project_data)
    add_template_file(f"{base_dir_files}/main.py",template_text)
  
    project_data["apps"][project_name]={"status":"upgraded"}
    del project_data["secret_key"]
    add_kamaqi_file(f"{str(project_path)}/kamaqi.json",project_data)

    os.chdir(project_path)

    if project_type=="docker":
        print("Creating docker image...")
        template=get_kamaqi_template("docker_file")
        template_text=template.render(**project_data)
        add_template_file(f"{project_path}/Dockerfile",template_text)

        template=get_kamaqi_template("docker_compose")
        template_text=template.render(**project_data)
        add_template_file(f"{project_path}/docker-compose.yaml",template_text)

        try:
            os.system("docker-compose stop")
            os.system("docker container prune --force")
            os.system("docker system prune --force")
            os.system(f"docker image rm {project_name.lower()}_image:latest")
        except:
            pass
        os.system(f"docker build  -t {project_name.lower()}_image .")

    if project_type=="normal":
        print(" Creating  a virtual environment ...")
        os.system("python3 -m venv env")
        os.system("source env/bin/activate")
        os.system("pip install -r requirements.txt")

    print(" Yor project was created successfully")
        
    



import os
from typer import prompt
from jinja2 import Environment, BaseLoader
from kamaqi.templates import app_auth,app_database,app_exceptions
from kamaqi.templates import app_schemas,app_router,app_exceptions
from kamaqi.templates import main,app_settings,env,requirements
from kamaqi.templates import router,schemas,models,crud,model
from kamaqi.templates import docker_file,docker_compose

APP_NAME="kamaqi"

environment = Environment(loader=BaseLoader)

def get_kamaqi_template(template_name:str):
    
    templates_dict={
        "app_router":app_router.APP_ROUTER,
        "app_settings":app_settings.APP_SETTINGS,
        "requirements":requirements.REQUIREMENTS,
        "app_auth":app_auth.APP_AUTH,
        "app_database":app_database.APP_DATABASE,
        "app_exceptions":app_exceptions.APP_EXCEPTIONS,
        "app_schemas":app_schemas.APP_SCHEMAS,
        "main":main.MAIN,
        "env":env.ENV,
        "router":router.ROUTER,
        "schemas":schemas.SCHEMAS,
        "models":models.MODELS,
        "model":model.MODEL,
        "crud":crud.CRUD,
        "docker_file":docker_file.DOCKER_FILE,
        "docker_compose":docker_compose.DOCKER_COMPOSE,

    }
    template=environment.from_string(templates_dict[template_name])
    return template

project_types={
    "1": "normal",
    "2": "docker"}

databases={
    "1": "MySQL",
    "2": "PostgreSQL",
    "3": "SQLite"}


def choose_project_type():
    print("Setting your project")
    print("Choose an option")
    print("[1] >> Create a  normal project")
    print("[2] >> Create a project with Docker")
    project_option = prompt("Your option is? ")
    os.system("clear")

    return project_types[project_option]

def choose_database_type(project_type):
    print("Setting your project")
    print("Choose an database option")

    posible_dbs=databases.copy()

    if project_type=="docker":
        del posible_dbs["3"]

    for key, vale in posible_dbs.items():
        print(f"[{key}] >> Use {vale}")

    db_option = prompt("Your option is? ")
    os.system("clear")

    return databases[db_option]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['kamaqi',
 'kamaqi.add',
 'kamaqi.app',
 'kamaqi.init',
 'kamaqi.remove',
 'kamaqi.run',
 'kamaqi.show',
 'kamaqi.templates',
 'kamaqi.upgrade',
 'kamaqi.utils']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=3.1.2,<4.0.0', 'typer[all]>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['kamaqi = kamaqi.main:app']}

setup_kwargs = {
    'name': 'kamaqi',
    'version': '0.1.3',
    'description': 'A comand line app',
    'long_description': '# Kamaqi\n\nA command line app for creating APIs with FastAPI, inspired in Artisan from Laravel and manage.py from Django.\n\n## The key features are:\n\n- Create a normal project and a project with `Docker`.\n- Choose a different databases  `MySQL`, `PostgreSQL` and `SQLite`.\n- Work as `Djando` creating apps.\n- Every app created with Kamaqi, contains a minimal CRUD.\n\n## Installation:\n\nInstall Kamaqi in the global environment\n\n```bash \npip install kamaqi\n```\n## Basic Usage:\n\n### Init your project:\n\nRun\n```bash\nkamaqi init project you_project_name\n```\nAdd choose the options, for setting your project.\n\n### Run your project\nRun \n```bash\ncd your_project_name\n```\n```bash\nkamaqi run project you_project_name\n```\n- Exploe your API documentation\n- For default kamaqi use the port 8000\n- Open in your browser http://localhost:8000/docs\n### Add apps to your project\nRun \n```bash\nkamaqi add apps users products shops ... etc\n```\n### Create files for your apps\nRun \n```bash\nKamaqi upgrade apps \n```\n- Refresh your editor files.\n- Refresh your API documentation\n\n## Project Status\n- The project currently is development and will be\nbugs.\n\n- Your can contribute to this project, repoting bugs, writing documentation, writing tests, with pull requests ... etc.\n\nFor more information visit [github repository](https://github.com/Mitchell-Mirano/kamaqi)\n\n\n\n\n\n',
    'author': 'Mitchell Mirano',
    'author_email': 'mitchellmirano25@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
